"""Module for working with the config file."""

import os
import shutil

from icecream import ic
import pandas as pd

from censuswrangler._schemas import config_schema


class Config:
    """Class for accessing the config file, providing useful attributes and checking the validity of the configuration."""

    def __init__(self, config_path: str):
        """Initialises the Config class object.

        Args:
            config_path (str): The file path of the config csv.
        """
        # Check if the config file exists then read it into a DataFrame
        self.config_path: str = config_path
        self.config_path_abs: str = os.path.abspath(config_path)
        assert os.path.exists(config_path), f"Config file not found at: {config_path}"
        self.df: pd.DataFrame = config_schema(pd.read_csv(config_path))

        # Cast all columns as strings
        self.df = self.df.astype(str)

        # Count rows
        self.row_count: int = len(self.df)
        self.unique_row_count: int = len(self.df.drop_duplicates())

        # Assert that there are no duplicate rows
        if self.row_count != self.unique_row_count:
            print("Duplicate rows found in config file, these will be ignored")

        # Get unique values for each column
        self.unique_datapackfile: list = self.df["DATAPACKFILE"].unique().tolist()
        self.unique_datapackfile_count: int = len(self.unique_datapackfile)

        self.unique_short: list = self.df["SHORT"].unique().tolist()
        self.unique_short_count: int = len(self.unique_short)

        self.unique_long: list = self.df["LONG"].unique().tolist()
        self.unique_long_count: int = len(self.unique_long)

        self.unique_custom_description: list = (
            self.df["CUSTOM_DESCRIPTION"].unique().tolist()
        )
        self.unique_custom_description_count: int = len(self.unique_custom_description)

        self.unique_custom_group: list = self.df["CUSTOM_GROUP"].unique().tolist()
        self.unique_custom_group_count: int = len(self.unique_custom_group)

    def summary(self):
        """Prints a summary of the config file"""
        ln = "-" * 50
        print(ln)
        print("Config file summary")
        print(ln)
        ic(self.config_path)
        ic(self.config_path_abs)
        ic(self.row_count)
        ic(self.unique_row_count)
        print(ln)
        ic(self.unique_datapackfile_count)
        ic(self.unique_short_count)
        ic(self.unique_long_count)
        ic(self.unique_custom_description_count)
        ic(self.unique_custom_group_count)
        print(ln)
        ic(self.unique_datapackfile)
        ic(self.unique_short)
        ic(self.unique_long)
        ic(self.unique_custom_description)
        ic(self.unique_custom_group)
        print(ln)


def create_config_template(
    output_folder: str, file_name: str = "censuswrangler_config"
) -> None:
    """Create a template config csv for use with the census

    Args:
        output_folder (str): The folder path where the config file will be created.
        file_name (str, optional): The name of the config file, excluding file type. Defaults to "censuswrangler_config".
    """
    assert os.path.isdir(output_folder), (
        f"The provided folder_path argument '{output_folder}' is not a directory or does not exist."
    )
    template_source = "censuswrangler/config_template.csv"
    output_path = os.path.join(output_folder, file_name + ".csv")
    output_path_abs = os.path.abspath(output_path)
    assert not os.path.exists(output_path), (
        f"File '{output_path_abs}' already exists, file creation aborted."
    )
    shutil.copy(template_source, output_path)
    print(f"Successfully created censuswrangler config template: '{output_path_abs}'")


if __name__ == "__main__":
    from icecream import ic

    # # Example usage with the config template
    # config = Config("censuswrangler/config_template.csv")
    # config.summary()
    create_config_template("test_output")



"""Defines the Data class, used for for working with and accessing the census datapack folder."""

import os

from icecream import ic

from censuswrangler.config import Config


class Data:
    """Class for accessing selections of the data inside a datapack. Prepares needed reference info per the details in the provided Config object."""

    def __init__(self, folder_path: str, geo_type: str, config: Config):
        """Initialises the Data class.

        Args:
            folder_path (str): The path of the data folder inside the datapack.
            geo_type (str): The geo type to access e.g. (LGA, SA1, SA2, etc).
            config (Config): The Config object containing config information.
        """
        self.folder_path = folder_path

        # Build a dictionary containing information of the files in the census datapack folder
        # Each entry represents a file code / file
        datapack_details = []
        for root, directories, files in os.walk(folder_path):
            for filename in files:
                file_path = os.path.join(root, filename)
                # Split the filename into its name and type components
                name, file_type = os.path.splitext(filename)
                # Split the name by '_'
                name_parts = name.split("_")
                # Split the path into its components
                path_parts = os.path.split(file_path)
                # Create a dictionary with the path components as key-value pairs
                file_dict = {
                    "filename": name,
                    "nameparts": {
                        "census_desc": name_parts[0],
                        "file_code": name_parts[1],
                        "country": name_parts[2],
                        "geo_type": name_parts[3],
                    },
                    "filetype": file_type,
                    "directory": path_parts[0],
                    "full_path": file_path,
                }
                # Add the dictionary to the list
                datapack_details.append(file_dict)
                # Filter the list to only include the target geo_type and data file codes
                datapack_details = [
                    file_info_dict
                    for file_info_dict in datapack_details
                    if (
                        file_info_dict["nameparts"]["geo_type"] == geo_type
                        and file_info_dict["nameparts"]["file_code"]
                        in config.unique_datapackfile
                        and file_info_dict["filetype"] == ".csv"
                    )
                ]
        self.details = datapack_details

    def summary(self):
        """Prints a summary of the datapack selection"""
        ic(datapack.details)


if __name__ == "__main__":
    folder_path = r"E:/Data/2021_GCP_all_for_AUS_short-header/2021 Census GCP All Geographies for AUS"
    config = Config("censuswrangler/config_template.csv")
    datapack = Data(folder_path, "LGA", config)
    datapack.summary()
